package eu.mhsl.minecraft.pixelblocks.pixelblock;

import eu.mhsl.minecraft.pixelblocks.Main;
import eu.mhsl.minecraft.pixelblocks.utils.MinMaxUtil;
import org.bukkit.Location;
import org.bukkit.NamespacedKey;
import org.bukkit.entity.EntityType;
import org.bukkit.entity.Interaction;
import org.bukkit.persistence.PersistentDataType;

import java.util.List;
import java.util.Objects;
import java.util.UUID;

public class PixelBlockHitbox {
    private static final NamespacedKey hitboxOfTag = new NamespacedKey(Main.plugin, "hitbox_of");

    private final Interaction interaction;

    public static PixelBlockHitbox fromExisting(PixelBlock parentBlock) {
        Interaction hitBox = parentBlock.getPixelBlockLocation().getNearbyEntitiesByType(Interaction.class, 1)
            .stream()
            .filter(interaction -> interaction.getPersistentDataContainer().has(hitboxOfTag))
            .filter(interaction -> Objects.equals(
                interaction.getPersistentDataContainer().get(hitboxOfTag, PersistentDataType.STRING),
                parentBlock.getBlockUUID().toString()
            ))
            .reduce((a, b) -> {
                throw new IllegalStateException(String.format("Mehrere hitboxen für PixelBlock '%s' gefunden!", parentBlock.getBlockUUID()));
            })
            .orElseThrow();

        return new PixelBlockHitbox(hitBox);
    }

    public static PixelBlockHitbox newHitbox(PixelBlock parentBlock) {
        return new PixelBlockHitbox(parentBlock.getPixelBlockLocation(), parentBlock.getPixels(), parentBlock.getBlockUUID());
    }

    private PixelBlockHitbox(Interaction interaction) {
        this.interaction = interaction;
    }

    private PixelBlockHitbox(Location pixelBlockLocation, List<Pixel> pixels, UUID parentBlockUUID) {
        float offset = (float) Main.configuration.hitboxOffset();
        int pixelsPerBlock = Main.configuration.pixelsPerBlock();

        if(pixels.size() > 5 || true) {
            interaction = (Interaction) pixelBlockLocation.getWorld().spawnEntity(
                pixelBlockLocation.clone().add(0.5, -offset, 0.5),
                EntityType.INTERACTION
            );
            interaction.setInteractionHeight(1 + 2*offset);
            interaction.setInteractionWidth(1 + 2*offset);
        } else {
            double startingX = MinMaxUtil.getMinProperty(pixels, pixel -> pixel.location.getX());
            double startingY = MinMaxUtil.getMinProperty(pixels, pixel -> pixel.location.getY());
            double startingZ = MinMaxUtil.getMinProperty(pixels, pixel -> pixel.location.getZ());

            double endingX = MinMaxUtil.getMaxProperty(pixels, pixel -> pixel.location.getX());
            double endingY = MinMaxUtil.getMaxProperty(pixels, pixel -> pixel.location.getY());
            double endingZ = MinMaxUtil.getMaxProperty(pixels, pixel -> pixel.location.getZ());

            Location spawnLocation = pixelBlockLocation.clone().add(
                ((startingX+endingX+1)/2)/pixelsPerBlock,
                (startingY/pixelsPerBlock)-offset,
                ((startingZ+endingZ+1)/2)/pixelsPerBlock
            );

            float height = (float) (endingY-startingY+1)/pixelsPerBlock + 2*offset;

            float width;
            if((endingX-startingX) > (endingZ-startingZ)) {
                width = (float) (endingX-startingX+1)/pixelsPerBlock + 2*offset;
            } else {
                width = (float) (endingZ-startingZ+1)/pixelsPerBlock + 2*offset;
            }

            if(spawnLocation.getX()+width/2 > pixelBlockLocation.getX()+1) {
                spawnLocation.subtract((spawnLocation.getX()+width/2)-(pixelBlockLocation.getX()+1), 0, 0);
            }
            if(spawnLocation.getX()-width/2 < pixelBlockLocation.getX()) {
                spawnLocation.add(pixelBlockLocation.getX()-(spawnLocation.getX()-width/2), 0, 0);
            }

            if(spawnLocation.getZ()+width/2 > pixelBlockLocation.getZ()+1) {
                spawnLocation.subtract(0, 0, (spawnLocation.getZ()+width/2)-(pixelBlockLocation.getZ()+1));
            }
            if(spawnLocation.getZ()-width/2 < pixelBlockLocation.getZ()) {
                spawnLocation.add(0, 0, pixelBlockLocation.getZ()-(spawnLocation.getZ()-width/2));
            }

            interaction = (Interaction) pixelBlockLocation.getWorld().spawnEntity(
                spawnLocation,
                EntityType.INTERACTION
            );

            interaction.setInteractionHeight(height);
            interaction.setInteractionWidth(width);
        }

        this.interaction.getPersistentDataContainer()
            .set(hitboxOfTag, PersistentDataType.STRING, parentBlockUUID.toString());
    }

    public void destroy() {
        this.interaction.remove();
    }
}
